/* ***********************************************************************************************************************
 *	Das Modul beinhaltet Funktionen, Hilfsvariablen und Konstanten 
 *	zur Berechnung der Mondpositionen.
 *	Die Funktionen und Algorithmen haben keinen Anspruch auf
 *	hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module contains functions, auxiliary variables and constants 
 * for the calculation of the moon positions. 
 * The functions and algorithms have no claim to 
 * high accuracy, completeness and correctness.
 *
 *	Funktionsliste- Function list:
 *
 * - moonAverageLengthL()													- 	Die durchschnittliche Lnge l des Mondes in Grad
 *																									The average length l of the moon in degrees
 *	- moonAverageLengthN()													-	Die durchschnittliche Lnge N des Mondes in Grad
 *																									The average length N of the moon in degrees
 *	- moonAverageAnomaly()													- 	Die durchschnittliche Anomalie des Mondes in Grad
 *																									The average anomaly of the moon in degrees
 *	- correctionsForEvection()													- 	Korrektur der Evektion des Mondes fr die Korrektur
 *  																		  						der durchschnittlichen Anomalie des Mondes in Grad
 *																									Correction of the evection of the moon for the correction
 *																									of the average anomaly of the moon in degrees
 * - yearEquation()																	- 	Die Korrektur fr die Jahresgleichung Ae	
 *																									The correction for the annual equation Ae
 * - a3()																					- 	Eine dritte Korrektur
 * 																									A third correction
 * - moonAverageAnomalyCorrected()									- 	Die durchschnittliche korregierte Anomalie des Mondes
 * 																									The average corrected anomaly of the moon
 * - equationOfMiddle()															- 	Die Korrektur fr die Gleichung der Mitte
 * 																									The correction for the equation of the center
 * - a4()																					- 	Weitere Korrektur fr die Lnge des Mondes
 *																								- 	Further correction for the length of the moon
 * - correctedLengthOfTheMoon()											- 	Der korrigierte Lngengrad der ekliptik des Mondes
 *																								- 	The function determines the corrected longitude of the moon
 * - variationV()																		- 	Der Korrekturwert fr die Variation V
 *																								- 	The function determines the correction variation (V)
 * - trueLongitudeOfTheMoon()												- 	Ermittelt den wahren Lngengrad des Mondes (L")
 *																									Determines the true length of the Moon (L)
 * - olderOfTheMoon()															- 	Ermittelt das Alter des Mondes D 
 *																									Determines the age of the moon D
 * - phaseOfTheMoon()															- 	Ermittelt die Mondphase mit den Werten von 0.0 - 1.0
 *																									Determines the moon phase with the values from 0. 0 - 1. 0
 * - correctedLongitudeOfTheMoon()										- 	Ermittelt den korrigierten Lngengrad des Knotens
 *																									Determines the corrected longitude of the node
 * - valueXYforEclipticLongitudeOfTheMoon()							- 	Ermittelt den ekliptischen X/Y-Wert des Lngengrades
 *																									Determines the averages the ecliptic X/Y value of the longitude
 * - eclipticLatitudeOfTheMoon()												- 	Ermittelt den ekliptischen Breitengrad des Mondes
 *																									Determines the ecliptic latitude of the moon
 * - findBrightLimbOfTheMoon()												- 	Ermittelt den Positionswinkel der hellen Seite des Mondes
 *																									Determines the position angle of the bright side of the moon
 * - writeMoonPhaseText()														- 	Schreibt die Mondphase in die Ausgabefelder
 *																									Writes the moon phase into the output fields
 * - rotateImage()																	-	Dreht ein Bild um X-Grad
 *																									Turn an image around X-degree
 *
 *
 *	07.10.2020, Dieter Steffan
 *
 **************************************************************************************************************************** */
 
 
// Konstanten zur Mondpositionsberechnung der Epoche 2010
// Constants to the lunar position calculation Epoch 2010
//
//const l0_moon = 91.929336;		// Mittlerer Lngengrad des Mondes der Epoche in Grad
const l0_moon = 91.92933599;	// Mittlerer Lngengrad des Mondes der Epoche in Grad
															// Mean longitude of the Moon of the Epoch in degrees
//const P0_moon = 130.143076;	// Mittlerer Lngengrad des Perigums der Epoche in Grad
const P0_moon = 130.1430763;	// Mittlerer Lngengrad des Perigums der Epoche in Grad
															// Mean longitude of the perigee of the epoch in degrees
//const N0_moon = 291.682547;	// Mittlere Lnge des Knotens in der Epoche 
const N0_moon = 291.6825466;	// Mittlere Lnge des Knotens in der Epoche 
															// mean longitude of the node at the epoch
const i_moon = 5.145396;				// Neigung der Mondumlaufbahn in Grad 
															// inclination of the Moons orbit in degrees
const e_moon = 0.0549;				// Exzentrizitt der Mondumlaufbahn
															// eccentricity of the Moons orbit
const a_moon = 384401;				// Halbe Hauptachse der Mondumlaufbahn in Km
															// semi major axis of the Moon's orbit in km
const theta0_moon = 0.5181;		// Mondwinkeldurchmesser in der Entfernung a_moon von der Erde in Grad
															// Moon's angular diameter at distance a from earth in degrees
const pi0_moon = 0.9507;				// Mondparallaxe in der Entfernung a_moon von der Erde
															// Moon's parallax at distance a_moon from the Earth in degrees 
 
// Hilfsvariablen
// Auxiliary variable
 
// Die Funktionen sin(), cos() 		... bentigen die Werte in Radiant, also
// Wert / RAD
// The functions sin () cos ()			... need the values in radiant, i.e.
// Value / RAD
// Die Funktionen asin(), acos() 		... bentigt die Werte dimensionslos.
// Das Ergebnis ist in Radiant und mu mit RAD multipliziert werden
// The functions asin () acos ()		... need the values dimensionless, i.e.
// After that you must multiple the resolution Value with RAD
var RAD = 180.0 / Math.PI;



/* **************************************************************************
	Die Funktion dreht ein Bild um X-Grad. Richtet sich nach dem
	Breitengrad an dem man sich gerade befindet.
						
	bergabewert:
			degRotate		=	Grad um die das Bild nach rechts gedreht wird als Number
			objTiny			=	Objekt des zu drehenden Bildes

	Rckgabewert:
			keiner
	---------------------------------------------------------------------------
	This function rotates an image by X degrees. It depends on 
	the latitude at which you are currently located. 
	
	Transfer value: 
	
			degRotate = Degree by which the image is rotated to the right as Number 
			objTiny 		= Object of the image to be rotated 
			
	Return value: 
			None
	************************************************************************* */
var zpos=0.0;
function rotateImage(degRotate, objTiny){
	// Nrdlich des quators haben wir 0 - 90 
	// Sdlich des quators haben wir 0 - -90
	// So bekommt man von Nord nach Sd eine
	// komplette Rotation um 180 des Bildes hin.
	// Dies entspricht der wahren Ansicht des Mondes.
	// Am quator liegt der Mond. Am Sdpol wechselt
	// die helle Seite und der Mond stellt sich auf den Kopf.
	// Also, geht man vom Norpol zum Sdpol, dann
	// rotiert der Mond einmal um 180.
	//
	// North of the equator we have 0 - 90 
	// South of the equator we have 0 - -90 
	// So you get from north to south a 
	// complete rotation around 180 of the picture. 
	// This corresponds to the true view of the moon. 
	// At the equator lies the moon. At the south pole 
	// the bright side changes and the moon turns upside down. 
	// So, if you go from the north pole to the south pole, then 
	// the moon rotates once by 180.
	zpos = 90.0 - degRotate - 0.0;
	// Fr die einzelnen Browserversionen ...
	// For the individual browser versions . . .
	objTiny.style.MsTransform = "rotate("+zpos+"deg)";
	objTiny.style.MozTransform = "rotate("+zpos+"deg) ";
	objTiny.style.OTransform = "rotate("+zpos+"deg)";
	objTiny.style.WebkitTransform = "rotate("+zpos+"deg)";
	objTiny.style.transform = "rotate("+zpos+"deg)";
	
}// End of rotateImage()



/* **************************************************************************
	Die Funktion ermittelt anhand des Mondphasenwertes, also dem Wert
	von 0.0 bis 1.0 und dem Mondalter in Grad positiv/negativ, die aktuelle
	Mondphase als Textbeschreibung und Bildinformation, die im Haupt-
	programm dargestellt wird und im externen Bildfenster zur Verfgung
	steht.
						
	bergabewert:
	moonage 				=	Das Mondalter in Grad und als Number
	phase 					=  Der Wert der Mondphase als Number
	objPicture				= Objekt der Bildausgabe
	comExtern			= Objekt des Broadcast Kanals
	geoPosErr			= Fehlercode bei der Ermittelung der Geo Daten
	geoPosStr				= Fehlermeldung bei der Ermittelung der Geo Daten

	Rckgabewert:
	TEXT					= Die Textbeschreibung der 4 Mondphasen
	---------------------------------------------------------------------------
	The function uses the moon phase value, i. e. the value from 0. 0 to 1. 0
	and the age of the moon in degrees positive/negative, to determine the
	current moon phase as a text description and image information, which 
	is displayed in the main program and is available in the external 
	image window.
	
	Transfer value:
	moonage 				= The age of the moon in degrees and as Number
	phase 					= The value of the moon phase as Number
	objPicture				= Object of image output
	comExtern 			= Object of the broadcast channel
	geoPosErr 			= Error code during the determination of the geo data
	geoPosStr 			= Error message during the determination of the geo data
	
	Return value:
	TEXT 					= The text description of the 4 moon phases
	************************************************************************* */
function writeMoonPhaseText(brightSide, phase, objPicture, comExtern, geoPosErr, geoPosStr){
	var PHASE = Number(phase);						// Die Phase des Mondes als Number
																					// The phase of the moon as a number
	var TEXT = "";														// Returnwert als Text als String
																					// Return value as text as string
	var PICNUMBER = 0;											// Variable fr die Bildnummer nach der Phase
																					// Variable fr die Bildnummer nach der Phase	
	var ret	;																	// Returnwert des Confirm Dialoges
																					// returnvalue of the confirm dialog
	var pathNorth = "";												// Pfad zu den Bildern auf der Nordhalbkugel
	var pathSouth = "";												// Pfad zu den Bildern auf der Sdhalbkugel
	var  XBRIGHTSIDE = Number(brightSide);	// Grad der hellsten Seite des Mondes
																					// Degrees of the bright limb of the moon
	// Beim Start werden die Geopositionen automatisch ermittelt. Wird dies vom Benutzer
	// blockiert, dann versuche es noch einmal, denn der Breitengrad wird bentigt, um
	// die Mondphase wird Nord- und Sdhalbkugel anzuzeigen.
	if (geoPosErr == 1 && geoPosWasLoaded == false){
		ret = confirm("Die Ermittlung der geografischen Positiondaten wurde von Ihnen mit folgender Meldung blockiert!\n\n'" +
								geoPosStr + 
								"'\n\nDas Programm bentigt aber diese Daten, um die Mondphase, fr die nrdliche- und sdliche Halbkugel richtig darstellen zu knnen!\n" +
								"Wenn Sie die Werte fr Lngen- und Breitengrad selbst eingetragen haben klicken Sie auf 'Abbrechen'\n" +
								"Wenn ich sie ermitteln soll auf 'OK'!");
		// Bei ret = true soll das Programm noch einmal die Daten ermitteln, weil der Benutzer keine
		// sinnvollen Angaben gemacht hat.
		// Bei ret = false hat der Benutzer die Daten selbst eingetragen.
		// If ret = true, the program should determine the data again, 
		// because the user has not made any meaningful entries.
		// If ret = false, the user has entered the data himself.
		if (ret)
			startGeoPosition(oLongitude, oLatitude, oAccuracy);
	}
	// Bildnummer aus der bergebenen Phasennummer  z.B.(0.456 ....) ermitteln
	// Image number from the given phase number e. g. B. (0. 456 . . . . )
	PICNUMBER = Math.floor(PHASE * 100.0);

	// Von Phase 0.997 - 1.0  haben wir Vollmond ...
	// From phase 0. 997 - 1. 0  we have full moon . . .
	if ((PHASE <= 1.0) && (PHASE >= 0.98) ){
	
		// Die aktuelle Monphase als Text ...
		// The current monphase as text . . .
		TEXT = "Vollmond";
		// Gebe die aktuelle Mondphase als kleines Bild im Hauptprogramm aus ...
		// Display the current moon phase as a small picture in the main program . . .
		objPicture.style.backgroundImage = "url(./Mondphasen/100.gif)";
		// Dreht das Bild nach dem Breitengrad in den richtigen Betrachtungswinkel von 90 bis -90 ...
		// Rotates the image according to latitude to the correct viewing angle from 90 to -90 . . .
		rotateImage(Number(oLatitude.value), objPicture);
		// Sende die Bilddaten an den Broadcast Kanal, das externe Bildfenster empfngt es wenn ntig ...
		// Send the image data to the broadcast channel, the external image window receives it if necessary . . .
		comExtern.postMessage("./Mondphasen/100.gif " + Number(oLatitude.value));
		
	// ... und Phase grer gleich 0.0 und kleiner 0.009  Neumond ...
	// . . . and phase greater than 0. 0 and less than 0. 009  New moon . . .
	}else if ((PHASE < 0.009) && (PHASE >= 0.0)){
	
		// Die aktuelle Monphase als Text ...
		// The current monphase as text . . .
		TEXT = "Neumond";
		// Gebe die aktuelle Mondphase als kleines Bild im Hauptprogramm aus ...
		// Display the current moon phase as a small picture in the main program . . .
		objPicture.style.backgroundImage = "url(./Mondphasen/0.gif)";
		// Dreht das Bild nach dem Breitengrad in den richtigen Betrachtungswinkel von 90 bis -90 ...
		// Rotates the image according to latitude to the correct viewing angle from 90 to -90 . . .
		rotateImage(Number(oLatitude.value), objPicture);
		// Sende die Bilddaten an den Broadcast Kanal, das externe Bildfenster empfngt es wenn ntig ...
		// Send the image data to the broadcast channel, the external image window receives it if necessary . . .
		comExtern.postMessage("./Mondphasen/0.gif " + Number(oLatitude.value));
		
	// ... liegt die Phase im Bereich 0.009 und 0.998 und die helle Seite des Mondes geht von 0 - 180 Grad, dann haben wir abnehmenden Mond ...
	// . . . the phase is in the range 0. 009 and 0. 998 and the bright limb of the moon is 0-180 degrees, then we have descending moon . . .
	}else if (((PHASE >  0.009)&& (PHASE <= 0.98) )&& (XBRIGHTSIDE >= 0.0) && (XBRIGHTSIDE <= 180.0)   ){
	
		// Die aktuelle Monphase als Text ...
		// The current monphase as text . . .
		TEXT = "Abnehmender Mond";
		// Gebe die aktuelle Mondphase als kleines Bild im Hauptprogramm aus ...
		// Display the current moon phase as a small picture in the main program . . .
		objPicture.style.backgroundImage = "url(./Mondphasen/Abnehmend/" +  PICNUMBER + ".gif)";
		// Dreht das Bild nach dem Breitengrad in den richtigen Betrachtungswinkel von 90 bis -90 ...
		// Rotates the image according to latitude to the correct viewing angle from 90 to -90 . . .
		rotateImage(Number(oLatitude.value), objPicture);
		// Sende die Bilddaten an den Broadcast Kanal, das externe Bildfenster empfngt es wenn ntig ...
		// Send the image data to the broadcast channel, the external image window receives it if necessary . . .
		comExtern.postMessage("./Mondphasen/Abnehmend/" +  PICNUMBER + ".gif " + Number(oLatitude.value));
		
	// und ... liegt die Phase im Bereich 0.009 und 0.998 und die helle Seite des Mondes geht von 180 - 360 Grad, dann haben wir zunehmenden Mond ...
	// and . . . the phase is in the range 0. 009 and 0. 998 and the bright limb of the moon is 180-360 degrees, then we have increasing moon . . .
	}else if (((PHASE >  0.009)&& (PHASE <= 0.98) )&& (XBRIGHTSIDE >= 180.0) && (XBRIGHTSIDE <= 360.0) ){
		// Die aktuelle Monphase als Text ...
		// The current monphase as text . . .
		TEXT = "Zunehmender Mond";
		// Gebe die aktuelle Mondphase als kleines Bild im Hauptprogramm aus ...
		// Display the current moon phase as a small picture in the main program . . .
		objPicture.style.backgroundImage = "url(./Mondphasen/Zunehmend/" +  PICNUMBER + ".gif)";
		// Dreht das Bild nach dem Breitengrad in den richtigen Betrachtungswinkel von 90 bis -90 ...
		// Rotates the image according to latitude to the correct viewing angle from 90 to -90 . . .
		rotateImage(Number(oLatitude.value), objPicture);
		// Sende die Bilddaten an den Broadcast Kanal, das externe Bildfenster empfngt es wenn ntig ...
		// Send the image data to the broadcast channel, the external image window receives it if necessary . . .
		comExtern.postMessage("./Mondphasen/Zunehmend/" +  PICNUMBER + ".gif " + Number(oLatitude.value));
		
	// ... und fr alle nicht abgedeckten Flle.
	// . . . and for all uncovered cases.
	}else{
		TEXT = "N/A";
		objPicture.style.backgroundImage = "url(./Mondphasen/NA.gif)";
		comExtern.postMessage("./Mondphasen/NA.gif");
	}
	// Gebe TEXT an den Aufrufer zurck
	// Return TEXT to the caller
	return(TEXT);
	
}// End of writeMoonPhaseText()



/* **************************************************************************
	Die Funktion ermittelt den Positionswinkel des hellsten Gliedes 
	des Mondes	in Grad. Gerechnet wird vom Nordpol des Mondes 
	nach Osten.
	
	bergabewert:
	lambdasun			=	Ekliptischer Lngengrad der Sonne in Grad 
	deltasun				=	Ekliptischer Breitengrad der Sonne
	alphamoon			=	Ekliptischer Lngengrad des Mondes
	deltamoon			=	Ekliptischer Breitengrad des Mondes
	radiant				=	Umrechnung von RAD in Grad (180/PI)
	
	Rckgabewert:
	X						= Der Positionswinkel in Grad und Number
	---------------------------------------------------------------------------
	The function determines the position angle of the brightest l
	imb of the moon in degrees. Calculations are made from 
	the north pole of the moon to the east. 
	
	Transfer value: 
	lambdasun = Ecliptic longitude of the sun in degrees 
	deltasun = Ecliptic latitude of the sun 
	alphamoon = Ecliptic longitude of the moon 
	deltamoon = Ecliptic latitude of the moon 
	radiant = Conversion from RAD to degrees (180/PI) 
	
	Return value: 
	X = The position angle in degrees and number
	************************************************************************* */
function findBrightLimbOfTheMoon(rasun, decsun, ramoon, decmoon, radiant){
	var RADIANT = Number (radiant);					// Umrechnung von RAD in Grad
																					// Conversion from RAD to degrees
	var RASUN = Number(rasun);						// Ekliptischer Lngengrad der Sonne in Grad
																					// Ecliptic longitude of the sun in degrees
	var DECSUN = Number(decsun);					// Ekliptischer Breitengrad der Sonne
																					// Ecliptic latitude of the sun in degrees
	var RAMOON = Number(ramoon);				// Ekliptischer Lngengrad des Mondes
																					// Ecliptic longitude of the moon in degrees
	var DECMOON = Number(decmoon);			// Ekliptischer Breitengrad des Mondes
																					// Ecliptic latitude of the moon in degrees
	var X = 0.0;															// Ergebniswert fr X
																					// Result value for X
	var y = 0.0;															// Zhler der Division
																					// Counter of the Division
	var x = 0.0;															// Nenner der Division
																					// Denominator of the division
	// Berechne den y-Wert ...
	// Calculate the y-value. . .
	y = Math.cos(DECSUN / RADIANT)* Math.sin((RASUN - RAMOON)/ RADIANT);
	// ... Berechne den x-Wert
	// ... calculate the x-value. . .
	x = (Math.cos(DECMOON / RADIANT)* Math.sin(DECSUN / RADIANT)) - 
		  (Math.sin(DECMOON / RADIANT)* Math.cos(DECSUN / RADIANT) * (Math.cos((RASUN/ RADIANT) - (RAMOON/ RADIANT))));
	// ... dann den Gradwert des arcustangens von y/x
	//. . . then the degree value of the arcustangens of y/x division
	//X = Math.atan(y/x)* RADIANT;
	X = anotherArcTan(x, y)* RADIANT;
	// Bringe die Werte in den richtigen Bereich des Gradkreises
	// Bring the values to the correct range of the degree circle
	//X = unwindDeg(X);
	// Gebe X an den Aufrufer zurck
	// Return X to the caller
	return(X);
	
}// End of findBrightLimbOfTheMoon()



/* **************************************************************************
	Die Funktion ermittelt den ekliptischen Breitengradgrad des Mondes
	aus l", dem wahren ekliptischen Lngengrad und N', dem korregierten
	ekliptischen Langengrads des Knotens sowie einer Konstante i_moon, 
	hier im Modul definiert, und bringt die Daten in den richtigen Quadranten.
						
	bergabewert:
	lstrichstrich 		=	der ekliptischen Lngengrad in Grad und als Number
	nstrich 				=  Den korregierten ekliptischen Langengrads des Knotens
									als Number
	i					 		= Konstante als Number hier definiert
	
	Rckgabewert:
	Beta mond			= Der quatoriale Breitengrad des Mondes als
									Grad und Number
	--------------------------------------------------------------------------------------
	The function determines the ecliptic latitude of the moon from "l", the true
	ecliptic longitude and N', the corrected ecliptic longitude of the node and
	a constant i_moon, defined here in the module, and places the data in the
	correct quadrant. 
	
	Transfer value: 
	lstrichstrich 						= the ecliptic longitude in degrees and as number 
	nstrich	 								= the corrected ecliptic longitude of the node as number
	i 											= constant as number defined here
	
	Return value: beta moon 	= the equatorial latitude of the moon as degree and number
	
	************************************************************************* */
function eclipticLatitudeOfTheMoon(lstrichstrich, nstrich, i){
	var NSTRICH = Number(nstrich);						// Der korregierte durchschnittliche Knoten
																				// The corrected longitude of the node
	var LSTRICHSTRICH = Number(lstrichstrich); 	// Der korrigierte ekliptische Lngengrad des Mondes
																				// The corrected ecliptic longitude of the moon
	var IMOON = Number(i);									// Definierte Konstante im Modul
																				// Defined constant in the module
	var BETAM = 0.0;													// Der Returnwert
																				//  The return value
	// Berechnet Beta m
	// Calculating Beta m
	BETAM = Math.asin(Math.sin((LSTRICHSTRICH - NSTRICH) / RAD) *  Math.sin(IMOON / RAD))* RAD;
	// Gebe Beta m an den Aufrufer zurck
	// Return beta m to the caller
	return(BETAM);
}



/* ******************************************************
	Die Funktion ermittelt den quatorialen Lngengrad des Mondes
	aus den ekliptischen Koordinaten X und Y und bringt ihn in den
	richtigen Quadranten.
						
	bergabewert:
	x 	=	Ekliptische X-Koordinate als Number
	y 	=  Ekliptische Y-Koordinate als Number
							
	Rckgabewert:
	Lambda mond	= 	Der quatoriale Lngengrad des Mondes als
								Grad und Number
	-----------------------------------------------------------------------------
	The function determines the equatorial longitude of the moon
	from the ecliptic coordinates X and Y and brings him into the
	right quadrants.
	
	Transfer value:
	x = Ecliptic X coordinates as number
	y = Ecliptic Y-coordinate as number

	Return value:
	Lambda mond = The equatorial longitude of the moon as
	degrees and Number
	
	*************************************************** */
function eclipticLongitudeOfTheMoon(x, y, nstrich){
	var X = Number(x);							// Den berechneten Wert x in der Ekliptik als Number
																// The calculated value x in the ecliptic as number
	var Y = Number(y);							// Den berechneten Wert y in der Ekliptik als Number
																//The calculated value y in the ecliptic as number
	var NSTRICH = Number(nstrich);		// Der korregierte durchschnittliche Knoten
																// The corrected longitude of the node
	var ARCTANXY = 0.0;							// Der ekliptische Lngengrad des Mondes in Grad
																// und als Number
																// The ecliptic longitude of the moon in degrees and
																// as number
	// Der arcus Tan von Y/X in Radiant x 180/PI = Grad
	//// The arcus tan of Y/X in radiant x 180/PI = degrees
	ARCTANXY = anotherArcTan(X, Y)* RAD;
	ARCTANXY += NSTRICH;
	// ... und dann noch in den richtigen Quadranten bringen ...
	// ... and then bring it into the right quadrants ...
	ARCTANXY = unwindDeg(ARCTANXY);
	// ... und zurck an den Caller.
	// ... Return ecliptic longitude of the Moon to the caller
	return(ARCTANXY);
	
}// End of eclipticLongitudeOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt den ekliptischen X/Y-Wert des
	Lngengrades des Mondes als Number.
						
	bergabewert:
	lstrichstrich 			=	Der korrigierte ekliptische Lngengrad des Mondes
										in Grad als Number.

	nstrich 					=  Der korrigierte ekliptische Lngengrad des Knotens
										in Grad und als Number.
							
	Rckgabewert:
	XY						= 	Die X/Y-Werte zur Berechnung der ekliptischen Lnge
									des Mondes als Number.
	-----------------------------------------------------------------------------
	The function determines the ecliptic X/Y value of the
	Longitude of the moon as number.
	
	Transfer value:
	lstrichstrich				= The corrected ecliptic longitude of the moon
	in degrees as number. 
	
	nstrich 						= The corrected ecliptic longitude of the node
										in degrees and as numbers.
										
	Return value:
	XY 							= The X/Y values used to calculate the ecliptic length
										of the moon as a number.
	
	*************************************************** */
function valueXYforEclipticLongitudeOfTheMoon(lstrichstrich, nstrich){
	var LSTRICHSTRICH = Number(lstrichstrich); 	// Der korrigierte ekliptische Lngengrad des Mondes
																				// The corrected ecliptic longitude of the moon
	var NSTRICH = Number(nstrich);						// Der korrigierte ekliptische Lngengrad des Knotens
																				// The corrected ecliptic longitude of the node
	var XY = [];															// Ergebnisarray - Result array
	
	// Berechnet den Y-Wert und speichert in an Position 0 ...
	// Calculates the Y-value and saves to position 0. . .
	XY.push(  Math.sin((LSTRICHSTRICH - NSTRICH)/RAD) * Math.cos(i_moon / RAD) );
	// ... und den X-Wert an Positon 1
	//. . . and the X value of Positon 1. . .
	XY.push( Math.cos( (LSTRICHSTRICH - NSTRICH) /RAD) );
	// Gebe XY an den Aufrufer zurck
	// Return XY to the caller
	return(XY);
	
}// End of valueXYforEclipticLongitudeOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt den korrigierten Lngengrad des Knotens
	aus dem durchschnittlichen Lngengrad des Mondes N und der
	mittleren Sonne.
						
	bergabewert:
	averageLongitudeMoon 			=	Durchschnittlicher Lngengrad
													des Mondes N als Grad und Number
	averageMiddleLongitudeSun 	=  Mittlere Sonne als Grad und Number
							
	Rckgabewert:
	N'			= Korrigierter Lngengrad des Knotens als Grad und Number
	-----------------------------------------------------------------------------
	The function determines the corrected longitude of the node
	from the average longitude of the moon N and the
	middle sun.
	
	Transfer value:
	averageLongitudeMoon 			= 	Average longitude
													of the moon N as degree and number
	averageMiddleLongitudeSun 	= Middle Sun as degree and number

	Return value:
	N' = Corrected longitude of the node as degree and number
	
	*************************************************** */
function correctedLongitudeOfTheMoon(averageLongitudeMoon, averageMiddleLongitudeSun){
	// Gebe N' an den Aufrufer zurck
	// Return N' to the caller
	return(Number(averageLongitudeMoon) -  (0.16 * Math.sin(Number(averageMiddleLongitudeSun )/ RAD)));
	
}// End of correctedLongitudeOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt die Mondphase mit den Werten von 0.0 - 1.0.
	Wobei 	1 					= Vollmond
					0 					= Neumond
	0.0 <= Phase <= 1.0	= Zunehmende Sichel oder Abnehmende Sichel
						
	bergabewert:
	olderOfMoon 				=	Das Alter des Mondes in Grad und Number
										Abnehmende Sichel negativ oder Alter + 360.
										Beide Werte sind equivalent zueinander.
										Zunehmende Sichel positiv.
							
	Rckgabewert:
	F						= 	Die Mondphase als Number mit den Werten
								0.0 <= F <= 1.0
	-----------------------------------------------------------------------------
	The function determines the moon phase with the values from 0. 0 - 1. 0.
	Where 	1 						= full moon
				0 						= New moon
	0. 0 <= Phase <= 1. 0 	= Increasing sickle or decreasing sickle

	Transfer value:
	olderOfMoon 					= The age of the moon in degrees and numbers
											Decreasing sickle negative or age + 360.
											Both values are equivalent to each other.
											Increasing sickle positive.
	Return value:
	F = The moon phase as a number with the values	0. 0 <= F <= 1. 0

	*************************************************** */

function phaseOfTheMoon(olderOfMoon){
	// Gebe F an den Aufrufer zurck
	// Return F to the caller
	return((0.5 * (1 - Math.cos(Number(olderOfMoon) / RAD))).toFixed(3));
	
}// End of phaseOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt das Alter eds Mondes D in Grad aus dem
	Lngengrad der Ekliptik der Sonne (Lambda sonne) in Grad und
	dem wahren Lngengrad der Mondumlaufbahn in Grad (l").
	
						
	bergabewert:
	lstrichstrich 				=	Der wahre Lngengrad der Mondumlauf
										-bahn in Grad und als Number.
	Lambda sonne			=	Lngengrad der Ekliptik der Sonne in Grad
										und als Number.
							
	Rckgabewert:
	D								= 	Alter des Mondes in Grad und als Number.
										Abnehmende Sichel negativ oder Alter + 360.
										Beide Werte sind equivalent zueinander.
										Zunehmende Sichel positiv.

	-----------------------------------------------------------------------------
	The function determines the age of eds Moon D in degrees from the
	Longitude of the ecliptic of the sun (Lambda sun) in degrees and
	the true longitude of the lunar orbit in degrees (l).
	
	Transfer value:
	il''							= The true longitude of the lunar orbit
									- track in degrees and as number.
	Lambda sun 			= Longitude of the ecliptic of the sun in degrees
									and as a number.
									
	Return value:
	D = 	Age of the moon in degrees and as number.
			Decreasing sickle negative or age + 360.
			Both values are equivalent to each other.
			Increasing sickle positive.

	*************************************************** */

function olderOfTheMoon(lstrichstrich, lambdasun){
	var age = Number(lstrichstrich) -  Number(lambdasun);
	// Bringe die Werte in den richtigen Bereich des Gradkreises
	// Bring the values to the correct range of the degree circle
	if (age < -180.0)age += 360.0;
	//age = unwindDeg(age);
	// Gebe D an den Aufrufer zurck
	// Return D to the caller
	return(age);
	
}// End of olderOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt den wahren Lngengrad des Mondes (L")
	in Grad und als Number.
	
						
	bergabewert:
	lstrich		 				=	Die korrigierte Lnge des Mondes (l')
										Grad und als Number.
	v								=	Die Korrektur der Variation (V) als Number.
							
	Rckgabewert:
	l"								= 	Der wahre Lngengrad des Mondes l" in
										Grad und als Number.

	-----------------------------------------------------------------------------
	The function determines the true longitude of the moon (L")
	in degrees and as numbers.
	
	Transfer value:
	lstrich						=	The corrected length of the moon (l')
										Degree and as a number.
	v 								= 	The correction of the variation (V) as a number.
	
	Return value:
	l" 								= 	The true longitude of the moon in
										Degree and as a number.

	*************************************************** */
function trueLongitudeOfTheMoon(lstrich, v){
	var LSTRICH = Number(lstrich);	// Der korregierte Lngengrad des Mondes
	var V = Number(v);							// Die Korrektur fr die Variation V des Lngengrades
	// Gebe l" an den Aufrufer zurck
	// Return l" to the caller
	return(LSTRICH + V);
	
}// End of trueLongitudeOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt die Korrektur Variation (V) zur Anwendung
	auf den Lngengrad des Mondes, um den wahren Lngengrad
	des Mondes zu ermitteln.
	
						
	bergabewert:
	lstrich		 				=	Die schon korrigierte Lnge des Mondes (l')
										Grad und als Number.
	msun2						=	Die mittlere Sonne in Grad und als Number.
							
	Rckgabewert:
	V								= 	Die Variation V als Number.

	-----------------------------------------------------------------------------
	The function determines the correction variation (V) for application
	to the longitude of the moon, to the true longitude
	of the moon.
	
	Transfer value:
	lstrich						= 	The already corrected length of the moon (l') in
										Degree and as a number.
	msun2 						= The middle sun in degrees and as number.
	
	Return value:
	V 								= The variation V as a number.

	*************************************************** */
function variationV(lstrich, msun2){
	var LSTRICH = Number(lstrich);	//Die schon korrigierte Lnge des Mondes
	var MSUN2 = Number(msun2);	// Die mittlere Sonne
	// Gebe V an den Aufrufer zurck
	// Return V to the caller
	return(0.6583 * Math.sin((2 * (LSTRICH - MSUN2)) / RAD));
	
}// End of variationV()




/* ******************************************************
	Die Funktion ermittelt den korregierten Lngengrad des Mondes
	(l') . Wird dann spter noch durch die Variation (V) zum wahren
	Lngengrad des Mondes ergnzt.
	
						
	bergabewert:
	l				 				=	Der durschnittliche Lngengrad des Mondes
										in Grad und als Number.
	Ev								=	Korrektur fr die Evektion als Number.
	Ec								= 	Korrektur fr die Gleichung der Mitte als
										Number.
	Ae								= Korrektur der Jahresgleichung als Number.
	a4								= Weitere Korrektur als Number.
							
	Rckgabewert:
	l'								= 	Der korrigierte Lngengrad des Mondes in
										Grad und als Number.

	-----------------------------------------------------------------------------
	The function determines the corrected longitude of the moon (l').
	It is then later supplemented by the variation (V) to the true
	longitude of the moon.
	
	Transfer value:
	l 								= The mean longitude of the moon
										in degrees and as numbers.
	Ev 							= 	Correction for the Evection as Number.
	Ec 							= 	Correction for the equation of the middle as number.
	Ae 							= 	Correction of the annual equation as a number.
	a4 							= 	Further correction as number.
	
	Return value:
	l' 								= 	The corrected longitude of the moon in degrees 
										and as number.

	*************************************************** */

function correctedLengthOfTheMoon(l, Ev, Ec, Ae,  a4){
	var L = Number(l);					// Durchschnittliche Lnge des Mondes
	var EV = Number(Ev);				// Korrektur fr Evektion
	var EC = Number(Ec);				// Korrektur Gleichung der Mitte
	var AE = Number(Ae);				// Korrektur der Jahresgleichung
	var A4 = Number(a4);				// Weitere Korrektur
	// Gebe l' an den Aufrufer zurck
	// Return l' to the caller
	return(L + EV + EC - AE + A4);
	
}// End of correctedLengthOfTheMoon()




/* ******************************************************
	Die Funktion ermittelt eine weitere Korrektur fr den korrigierten
	Lngengrad des Mondes, um den wahren Lngengrad des Mondes
	berechnen zu knnen. 
	
						
	bergabewert:
	mm				 				=	Die korrigierte Anomalie des Mondes
											in Grad und als Number.
							
	Rckgabewert:
	A4									= 	Der Korrekturwert als Number.

	-----------------------------------------------------------------------------
	The function determines another correction for the corrected
	Longitude of the moon to the true longitude of the moon
	to be able to calculate.
	
	Transfer value:
	mm 								= The corrected anomaly of the moon
											in degrees and as number.
	
	Return value:
	A4									= Correction value as number.

	*************************************************** */
function a4(mm){
	var MM = Number(mm);	// Die korregierte Anomalie des Mondes als Number
												// The corrected anomaly of the Moon as Number
	// Gebe A4 an den Aufrufer zurck
	// Return A4 to the caller
	return(0.214 * Math.sin((2 * MM) / RAD));
	
}// End of a4()




 /* ******************************************************
	Die Funktion ermittelt die Korrektur der 
	Gleichung der Mitte.
						
	bergabewert:
	mm_strich					= Die durchschnittliche korregierte
											Anamalie des Mondes
							
	Rckgabewert:
	mm_strich		= Korrektur fr die Gleichung der Mitte
	----------------------------------------------------
	The function determines the correction of the
	Equation of the middle.

	Transfer value:
	mm_strich = The average corrected
							Anamaly of the Moon

	Return value:
	Ec 				= correction for the equation of the middle
*************************************************** */
function equationOfMiddle(mmstrich){
	var MMSTRICH = Number(mmstrich);
	var V = 6.2886 * Math.sin(MMSTRICH / RAD);
	
	// Gebe Ec an den Aufrufer zurck
	// Return Ec to the caller
	return(V);
	
}// End of equationOfMiddle()




 /* ******************************************************
	Die Funktion ermittelt die korregierte durch-
	schnittliche Anomalie des Mondes in Grad.
						
	bergabewert:
	mm					= Mittlere Sonne in Grad als Number
	ev					= Die Korrektur fr die Evektion als Number
	ae					= Die Korrektur fr die Jahresgleichung
	a3					= Eine weitere Korrektur
						
	Rckgabewert:
	mm_strich		= Die korrigierte durchschnittliche Anomalie
							   des Mondes als Number
	----------------------------------------------------
	The function determines the corrected by-
	average anomaly of the moon in degrees

	Transfer value:
	mm = Mean sun in degrees as number
	ev = The correction for the evection as number
	ae = The correction for the annual equation
	a3 = Another correction

	Return value:
	mm_strich = The corrected average anomaly
							of the Moon as Number
*************************************************** */
function moonAverageAnomalyCorrected(mm, ev, ae, a3){
	var MM = Number(mm);	// Mittlere Sonne in Grad als Number
	var EV = Number(ev);		// Die Korrektur fr die Evektion als Number
	var AE = Number(ae);		// Die Korrektur fr die Jahresgleichung
	var A3 = Number(a3);		// Eine weitere Korrektur
	// Gebe Wert an den Aufrufer zurck
	// Return Value to the caller
	return(MM + EV - AE - A3);
	
}// End of moonAverageAnomalyCorrected()




 /* ******************************************************
	Die Funktion ermittelt die Korrektur A3 aus
	einer Konstante und Lambdasonne2
						
	bergabewert:
	Msun				= Mittlere Sonne in Grad als Number
						
	Rckgabewert:
	A3					= Jahresgleichung der mittleren Sonne als Number
	----------------------------------------------------
	The function determines the annual equation A3 from
	a constant and lambda-sonne2

	Transfer value:
	Msun 				= Middle Sun in degrees as Number

	Return value:
	A3 					= Correction A3 of the mean sun as number
*************************************************** */
function a3(Msun){
	var MSUN = Number(Msun);	// Die mittlere Sonne korregiert mit N als Number
														// The Middle Sun Corrects with N as Number
	// Gebe A3 an den Aufrufer zurck
	// Return Ae to the caller
	return(0.37 * Math.sin(MSUN / RAD));
	
}// End of a3()




 /* ******************************************************
	Die Funktion ermittelt die Jahresgleichung Ae aus
	einer Konstante und Lambdasonne2
						
	bergabewert:
	Msun				= Mittlere Sonne in Grad als Number
						
	Rckgabewert:
	Ae					= Jahresgleichung der mittleren Sonne als Number
	----------------------------------------------------
	The function determines the annual equation Ae from
	a constant and lambda-sonne2

	Transfer value:
	Msun 				= Middle Sun in degrees as Number

	Return value:
	Ae 				= Yearly equation of the mean sun as number
*************************************************** */
function yearEquation(Msun){
	var MSUN = Number(Msun);	// Die mittlere Sonne korregiert mit N als Number
														// The Middle Sun Corrects with N as Number
	//console.log("yearEquation Msun=" + MSUN);
	// Gebe Ae an den Aufrufer zurck
	// Return Ae to the caller
	return(0.1858 * Math.sin(MSUN / RAD));
	
}// End of yearEquation()




 /* ******************************************************
	Die Funktion ermittelt die Korrektur fr die Evektion
	des Mondes als Teil der der Korrektur fr die
	Anomalie des Mondes.
						
	bergabewert:
	Mm 						= 	Die durchschnittliche Anomalie des Mondes in Grad als Number
	l0							=  Die durschnittliche Lnge des Mondes in Grad als Number
	lambdaSonne	=	der Lngengrad der Ekliptik der Sonne in Grad als Number
						
	Rckgabewert:
	Die Korrektur Ev der Evektion des Mondes als Number in Grad
						
	----------------------------------------------------
	The function determines the correction for the evection
	of the moon as part of the correction for the
	Anomaly of the Moon.
	
	Transfer value:
	Mm 					= The average anomaly of the moon in degrees as number
	l0					 	= The mean length of the moon in degrees as number
	lambdaSun 	= the longitude of the ecliptic of the sun in degrees as number

	Return value:
	The Correction Ev of the Evection of the Moon as Number in Grades

*************************************************** */
function correctionsForEvection(Mm, l, lambdaSonne){
	var Ev = 0.0;											// Returnwert Deklarieren und als Number Initialisieren
	var MM = Number(Mm);						// Durchschnittliche Anomalie Deklarieren und als Number Initalisieren
	var LDSUN = Number(lambdaSonne);	// Lngengrad der Ekliptik der Sonne Deklarieren und als Number Initalisieren
	var L = Number(l);								// Durchschnittliche Lnge des Mondes Deklarieren und als Number Initalisieren
	// Hilfsvariable C
	var C = L - LDSUN;
	// Berechne die Korrekur
	// Calculating the correction
	Ev = 1.2739 * Math.sin(((2 *  C) -   MM) /  RAD );
	// Gebe Ev an den Aufrufer zurck
	// Return Ev to the caller
	return(Ev);
	
}// End of correctionsForEvection()




 /* ******************************************************
	Die Funktion ermittelt die durchschnittliche Lnge l
	des Mondes in Grad. Dabei wird die eine im Modul
	definierten Konstante l0_moon verwendet.
						
	bergabewert:
	countDays = 	Die Anzahl der Tage in der Epoche bis zum
							Berchnungsdatum
	l0					=  Konstante fr die Berechnung im Modul l0_moon
						
	Rckgabewert:
	Die mittlere Lnge des Mondes in Grad als Number
						
	----------------------------------------------------
	The function determines the average length l
	of the moon in degrees. The one in the module is
	defined constant l0_moon used.
	
	Transfer value:
	countDays = The number of days in the period up to
	Calculation date
	l0 = constant for the calculation in the module l0_moon

	Return value:
	The mean length of the moon in degrees as number

*************************************************** */
function moonAverageLengthL(countDays, l0){
	// Die durchschnittliche Lnge l des Mondes als Returnwert ...
	var l = 0.0;
	// ... und hier aus 2 Konstanten und Anzahl der Tage seit Beginn der Epoche berechnen ...
	l = 13.1763966 * Number(countDays) + Number(l0);
	// ... und in den positiven Bereich von 0 - 360 bringen ...
	//while(l > 360.0) l -= 360.0;
	//while(l < -180.0) l += 180.0;
	l = unwindDeg(l );
	// Gebe l an den Aufrufer zurck
	// Return l to the caller
	return(l);
	
}// End of moonAverageLengthL()




 /* ******************************************************
	Die Funktion ermittelt die durchschnittliche Lnge N
	des Mondes in Grad. Dabei wird die eine im Modul
	definierten Konstante N0_moon verwendet.
						
	bergabewert:
	countDays 		= 	Die Anzahl der Tage in der Epoche bis zum
								Berchnungsdatum
						
	Rckgabewert:
	Die mittlere Lnge des Mondes N in Grad als Number
						
	----------------------------------------------------
	The function determines the average length N
	of the moon in degrees. The one in the module is
	defined constant N0_moon used.
	
	Transfer value:
	countDays		 = The number of days in the period up to
								Calculation date

	Return value:
	The mean length of the moon in degrees as number

*************************************************** */
function moonAverageLengthN(countDays){
	var COUNTDAYS = Number(countDays);
	var ANM = 0.0;
	ANM = N0_moon - (0.0529539 *  COUNTDAYS);
	// ... und in den  Bereich von 0 - 360 bringen ...
	//. . . and bring to the range of 0  360. . .
	ANM  = unwindDeg(ANM );
	// Gebe N an den Aufrufer zurck
	// Return N to the caller
	return(ANM);
	
}// End of moonAverageLengthN()
 
 
 

 /* ******************************************************
	Die Funktion ermittelt die durchschnittliche Anomalie
	des Mondes in Grad. Dabei wird die eine im Modul
	definierten Konstante P0_moon verwendet.
						
	bergabewert:
	l			= Die mittlere Lnge des Mondes
	days	= 	Die Anzahl der Tage in der Epoche bis zum Berechnungsdatum
	
	Rckgabewert:
	Die mittlere Anomalie des Mondes in Grad als Number
						
	----------------------------------------------------
	The function determines the average anomaly
	of the moon in degrees. The one in the module is
	defined constant P0_moon used.
	
	Transfer value:
	l = The average length of the moon
	days = The number of days in the epoch up to the calculation date

	Return value:
	The mean anomaly of the moon in degrees as number

*************************************************** */
 function moonAverageAnomaly(l, days){
	var  L = Number(l);						// Die mittlere Lnge des Mondes
	var DAYS = Number(days);			// Die Anzahl der Tage in der Epoche bis zum Berechnungsdatum
	var MM = 0.0;									// Die mittlere Anomalie des Mondes
 
	// Berechnet die mittlere Anomalie aus der mittleren Lnge, der Anzahl der Tage in der Epoche bis zum Datum
	// und 2 Konstanten ...
	MM  =  L - (0.1114041 * DAYS) -  P0_moon;
 	// ... und in den  Bereich von 0 - 360 bringen ...
	while(MM > 360.0) MM -= 360.0;
	while(MM < -180.0) MM += 180.0;
	//MM = unwindDeg(MM);
 	// Gebe l an den Aufrufer zurck
	// Return l to the caller
	return(MM);
	
 }// End of moonAverageAnomaly()


 
 
 
 