/* ************************************************************************************
 *	Das Modul beinhaltet Funktionen zur Berechnung der Sonnenpositionen.
 *	Die Funktionen und Algorithmen haben keinen Anspruch auf
 *	hohe Genauigkeit, Vollstndigkeit und Richtigkeit.
 *
 * The module contains functions, auxiliary variables and constants 
 * for the calculation of the sunave no claim to 
 * high accuracy, completeness and correctness. 
 *
 *	Funktionsliste - Function list:
 *
 * - eccentricitySunEarthOrbit()							- Berechnet die Exzentrizitt des Erd Orbits als Ellipse.
 *																			- Calculate the eccentricity of the Earths orbit as an ellipse.
 * - meanEclipticLengthSun()								- Die mittlere ekliptische Lnge der Sonne der Epoche berechnen.
 *																			- Calculate the mean ecliptic longitude of the sun of the epoch.
 * - longitudeSunPerigaeum()								- Berechnet den Lngengrad der Sonne im Perigum.
 *																			- Calculate the longitude of the sun at perigee.
 * - mediumSun()												- Berechnet die mittlere Anomalie der Sonne bei kreisfrmiger
 *																			  Umlaufbahn der Erde um die Sonne.
 *																			- Calculates the mean anomaly of the sun at circular 
 * 																			   orbit of the earth around the sun.
 * - calcDaysSinceEpoch()									- Ermittelt die Tage von Epochenbeginn bis zum Berechnungsdatum
 *																			- Determines the days from the beginning of the epoch to the calculation date
 * - calculateN()													- Korregiert die Anzahl der Tage seit Epochenbeginn auf den Bereich 0 - 360
 *																			  Corregates the number of days since the beginning of the epoch to 0  360
 * - mediumSun2()												- Korregiert die mit der Funktion mediumSun()gefundene gesamt Gradzahl
 * 																			  Corrects the total degrees found with the function mediumSun()
 * - calculateEc()												- Berechnet eine Hilfswert
 *																			  Calculates a auxilary value
 * - lambdaSun()												- Berechnet die ekliptische Lnge der Sonne
 *																			  Calculates the ecliptic length of the Sun
 * - calcObliquityEcliptic()									- Berechnet die ekliptische Schiefe
 *																			  Calculates the ecliptic Obliquity
 * - decDegrees2AngleOfHours()						- Konvertiert dezimale Grad in Stundenwinkel als String
 *																			- Converts decimal degrees to hourly angles as string
 * - decDegrees2AngleOfTime()							- Konvertiert dezimale Grad in Stundenwinkel als Array
 *																			  Converts decimal degrees to hour angles as array
 * - angleOfTime2DecDegrees()							- Konvertiert ein Array mit Winkelzeiten in dezimale Grad
 *																			  Converts an array of angle times to decimal degrees
 * - convertAngelsecondsToDegrees()				- Konvertiert eine Winkelsekunde in dezimalen Wert
 *																			- Converts an angle second to decimal value
 * - convertDecdegreesToString()						- Konvertiert deszimale Grad ind , ', "
 *																			  Converts decimal degrees ind,', "
 * - calcNutation()												- Berechnet die Nutation der eklip. Lnge der Sonne und der Schiefe
 *																			  Calculates the nutation of the eclip. Length of the sun and the obliquity
 *
 *
 *
 *	24.09.2020, Dieter Steffan
 *
 *********************************************************************************** */
 
 
 // Feste Werte fr die Epoche 2010, die nicht berechnet werden mssen.
 // Fixed values for the epoch 2010, which do not have to be calculated.
 const epsilong 	= 279.557208;
 const omegag 	= 283.112438;
 const e				= 0.016705;
 const  r0				= 149598500;
 const theta0		= 0.533128;
 
 
 /* *********************************************************************************************
	Die Funktion berechnet die exzentrizitt des Orbits der Erde um die Sonne in einer
	Ellipse. Im Gegensatz dazu wird auch die mittlere Umlaufbahn verwendet, die den
	Kreisrunden Orbit annimmt.
	Karakteristische Gre fr Bahn eines Himmelskrpers und bezieht sich bei elliptischen
	Bahnen auf die groe Halbachse. Formelzeichen in der Astronomie ist e. Achtung, wird
	in der Mathematik als lineare Exzentrizitt verwende, in der Astronomie als nummerische
	Exzentrizitt.
	Venus	= 0,0068	-> Kleinste
	Merkur	= 0,2056	-> Grte
	Jenseits des Neptuns ist e grer, insbesondere die Kometen.
	Hailey	= 0,967
	Einige besitzen ein e von > 1.0 und sind damit hyperbolische (offene) Bahnen, 
	Wie z.B. 1I/?Oumuamua und kommen aus dem interstellaren Raum.
	
	Quelle:
	https://de.wikipedia.org/wiki/Exzentrizit%C3%A4t_(Astronomie)
	
	bergabewert:
	T							= 	Der Teil des vergangenen Jahrhunderts der Epoche
									als Number
							
	Rckgabewert:
	e							= 	Die Excentrizitt von 0.0 - 1.0 als Number
	----------------------------------------------------------------------------------------------------------
	The function calculates the eccentricity of the earth's orbit around the sun in an 
	ellipse. In contrast, the mean orbit is also used, which assumes the circular orbit.
	Characteristic quantity for the orbit of a celestial body and refers to the major semi-axis 
	in the case of elliptical orbits. Symbol in astronomy is e. Attention, is used in 
	mathematics as linear eccentricity, in astronomy as numerical eccentricity. 
	Venus 	= 0. 0068 -> Smallest 
	Mercury = 0. 2056 -> Largest 
	
	Beyond Neptune e is larger, especially the comets. 
	Hailey = 0. 967 
	Some have an e of > 1. 0 and are therefore hyperbolic (open) orbits, such as e.a.
	1I/?Oumuamua and come from interstellar space. 
	
	Source: 
	https://de. wikipedia. org/wiki/Exzentrizit%C3%A4t_(Astronomy)
	
	Transfer value: 
	T 							= The part of the last century of the epoch as Number 
	
	Return value: 
	e 							= The eccentricity from 0. 0 - 1. 0 as Number
	******************************************************************************************* */	
 function eccentricitySunEarthOrbit(T){
	return(0.01675104 - (0.0000418 * Number(T)) - (0.000000126 * Math.pow(Number(T), 2)));
	
}// End of eccentricitySunEarthOrbit()



/* *********************************************************************************************
	Die Funktion berechnet die Mittlere ekliptische Lnge der Sonne fr eine bestimmte
	Epoche 0. Januar 2010 0 Uhr, epsilon g = 279,557208 Grad. .Dies ist der Ausgangs-
	punkt fr die	Berechnungen und stellt die Position bei kreisrunder Umlaufbahn dar. 
	Von dort an werden die Gradzahlen hinzuadiert, die bisher durchlaufen wurden.
	Die Werte knnen auch negativ sein, wenn das Berechnungsdatum vor dem 
	Epochendatum liegt. Dann muss nur noch die elliptische Bewegung angemessen
	bercksichtigt werden, damit man die Position zum Berechnungsdatum findet.
		
	bergabewert:
	T						= 	Der Teil des vergangenen Jahrhunderts  
									T =(JD - 2415020.0)/ 36525
								seit dem 0. Januar 1900 12:00 Uhr
	
							
	Rckgabewert:
	Epsilon g			= 	Mittlerer Lngengrad der Sonne in  Grad und als Number
	----------------------------------------------------------------------------------------------------------
	The function calculates the mean ecliptic length of the Sun for a specific
	Epoch 0. January 2010 0 oclock, epsilon g = 279,557 208 degrees. This is the initial
	points for the calculations and represents the position at circular orbit. From there, the 
	grades that have been passed up to now are added. The values may also be negative 
	if the calculation date before the	Epoch date is. 
	Then only the elliptical movement has to be adequate	to be taken into account in order
	to find the position at the calculation date. 
	
	Transfer value:
	T 					= 	Part of the Last Century
								T = (JD  2 415 020. 0) / 36 525
							since the 0. January 1900 12:00


	Return value:
	Epsilon g 		= Mean longitude of the Sun in degrees and as number
	******************************************************************************************* */							
function meanEclipticLengthSun(T){
	return(279.6966778 + ((36000.76892/36525) * T) + (Math.pow(0.0003025, 2) ) );
	
}// End of meanEclipticLengthSun()




/* *********************************************************************************************
	Die Funktion berechnet den ekliptischen Lngengrad der Sonne am erdnchsten Punkts.
	Das Argument der Periapsis Omega ist eines der 6 Bahnelemente zur Lagebeschreibung
	einer Keplerelipse im Raum.
	Der Winkel legt die Orientierung der groen Halbachse (Apsis)fest. Gemessen wird er
	vom aufsteigenden Knoten an bis zur Periapsis.
	Ein Winkel von 0 bedeutet die grte Nhe zum Zentralkrper, wenn er die Ekliptik =
	Referenzebene von Sd nach Nord berschreitet.
	90 bedeuten, das die grte Nhe am nrdlichsten Punkt der Bahn erreicht wird. Je nach
	Zentralkrper sind verschieden Bezeichungen gebruchlich:
	
	- Argument des Perigums	-> Erde
	- Argument des Perihels		-> Sonne
	
	Die Lage der Periapsis kann auch durch Omega Strich beschrieben werden. Sie fhrt vom
	Frhlingspunkt (Widderpunkt) rechts entlang der Ekliptik bis zum aufsteigenden Knoten und von dort weiter
	entlang der Bahn bis zur Periapsis. Sie ist die Summe aus der Lnge des aufsteigenden 
	Knotens Omega und dem Argument der Periapsis omega
	
	Quelle:
	https://de.wikipedia.org/wiki/Argument_der_Periapsis
	https://de.wikipedia.org/wiki/Argument_der_Periapsis#/media/Datei:Bahnelemente.svg
	
	bergabewert:
	T						= 	Der Teil des vergangenen Jahrhunderts  
									T =(JD - 2415020.0)/ 36525
								seit dem 0. Januar 1900 12:00 Uhr

							
	Rckgabewert:
	Lngengrad der Periapsis (Perigum)omega Strich (g)	= 	In Grad und als Number
	----------------------------------------------------------------------------------------------------------
	The argument of the Periapsis Omega is one of the 6 orbital elements to describe the 
	position of a Kepler ellipse in space. The angle defines the orientation of the large semi-axis 
	(apse). It is measured from the ascending node to the periapsis. An angle of 0 means 
	the greatest proximity to the central body when it crosses the ecliptic = reference plane 
	from south to north. 90 means that the greatest proximity is reached at the northernmost 
	point of the track. Depending on the central body different names are in use: 
	
	- argument of perigee 	-> earth 
	- argument of perihel 	> sun
	
	
	The position of the periapsis can also be described by an omega line. It leads from the 
	vernal equinox (point of ram) on the right along the ecliptic to the ascending node and
	from there along the railway to the periapsis. It is the sum of the length of the ascending 
	node omega and the argument of Periapsis omega 
	
	Source: 
	https://de. wikipedia. org/wiki/Argument_der_Periapsis 
	https://de. wikipedia. org/wiki/Argument_der_Periapsis#/media/Datei:Bahnelemente. svg 
	
	Transfer value: 
							T = The part of the last century 
								T =(JD - 2415020. 0)/ 36525 
							since the 0. January 1900 12:00 
	
	Return value: 
	Longitude of the periapsis (perigee)omega dash (g) = In degrees and as number
	******************************************************************************************* */					
function longitudeSunPerigaeum(T){
		return(281.2208444 + (1.719175 * T) + (0.000452778 * Math.pow(T, 2)));
		
}// End of longitudeSunPerigaeum()



/* *********************************************************************************************
	Die Funktion berechnet die Mittlere Anomalie der Kepler Gleichung bei elliptischen
	Bahnen von Hinnelskrpern. Die kepler Gleichung wird bei den Zeitgleichungen
	angewendet. Eine Aufgabe ist es die wahre Anomalie der Erde auf der ellipt. Bahn
	um die Sonne zu ermitteln.
	Bei gleichfrmig verflieender Zeit bewegt sich ein Himmelskrper auf einer kreis-
	frmigen Umlaufbahn, die um die Ellipse gelegt wird, mit gleicher Winkelgeschwin-
	digkeit.
	Zu einem Zeitpunkt tp werden das Objekt auf der Kreisbahn, wie auch das Objekt
	auf der Ellipse, als in der Periapsis Z stehend angenommen. beide Punkte haben
	dieselbe Umlaufzeit und stehen zu einem ganzzahligen Vielfachen immer wieder
	in der Periapsis (Nchster Punkt zum Schwerpunkt)und bei halbzahligem 
	Vielfachen in der Apoapsis (Gegenber am 	Entferntesten Punkt zum Schwerpunkt).
	Die Winkel werden im Bogenma dargestellt, hier in Grad.
	Die momentage Lage des hypotetischen Objekts Y wird auf dem Hilfskreis als Winkel
	M zwischen dem Ellipsenmittelpunkt C und der Periapsis Z angegeben. Es ist die
	mittlere Anomalie.
	
	Quelle:
	https://de.wikipedia.org/wiki/Kepler-Gleichung#Mittlere_Anomalie
	
	bergabewert:
	D													= 	Tage seit Jahresbeginn als Integer
							
	Rckgabewert:
	Mittlere Anomalie Sonne				= 	Als gesamte Grad seit Anfang der Epoche
															und als Number
	----------------------------------------------------------------------------------------------------------
	The function calculates the Mean Anomaly of the Kepler equation for elliptical 
	orbits of Hinnel bodies. The Kepler equation is applied to the equations of time. 
	One task is to find the true anomaly of the earth on the elliptical. orbit to 
	determine the sun. When time passes uniformly, a celestial body moves 
	on a circular orbit around the ellipse at the same angular velocity. At a time tp, 
	the object on the circular orbit, as well as the object on the ellipse, are assumed 
	to be in the periapsis Z. both points have the same orbital period and are always 
	in the periapsis (nearest point to the center of gravity) for an integer multiple 
	and in the apoapsis (opposite the farthest point from the center of gravity) 
	for a half integer multiple. The angles are displayed in radians, here in degrees.
	The momentary position of the hypotic object Y is given on the auxiliary 
	circle as the angle M between the center of the ellipse C and the periapsis 
	Z. It's the mean anomaly. 
	
	Source: https://de. wikipedia. org/wiki/Kepler-Gleichung#Mittlere_Anomalie 
	
	Transfer value: 
	D 								= Days since the beginning of the year as integer 
	
	Return value: 
	Mean anomalie Sun 	= As a total degree since the beginning of the epoch 
									   and as Number
	******************************************************************************************* */						
function mediumSun(D){
	var DAYS = Number(D);					// Tage seit der Epoche
																	// Days since the epoch
	// Die mittlere Sonne bei kreisfrmiger Umlaufbahn zurck an den Aufrufer.
	// The middle sun in a circular orbit back to the caller.
	return( ((360 / 365.242191) * DAYS)); 
	
}// End of mediumSun()
					
	

 /* *********************************************************************************************
	Die Funktion berechnet die Anzahl der Tage ab dem Beginn einer Epoche, also z.B.
	der Epoche 0.5 Januar 1900 bis zum aktuellen Berechnungsdatum. Ein Datum das
	vor dem Epochenbeginn liegt wird negativ berechnet und muss auch so bleiben.
	
	bergabewert:
	yearEpoch			= Das Jahr der Epoche, also z.B. 1900 als Integer 
	actualDay				= Berechnungstag als Integer
	actualMonth			= Berechnungsmonat als Integer
	actualYear			= Berechnungsjahr als Integer
							
	Rckgabewert:
	days						= 	Anzahl der Tage ab Epochenbeginn
	false						= Im Fehlerfalle
	----------------------------------------------------------------------------------------------------------
	The function calculates the number of days from the beginning of an epoch, 
	i. e. z. B. the epoch 0. 5 January 1900 up to the current calculation date. 
	A date that lies before the beginning of an epoch is calculated negatively 
	and must remain so. 
	
	Transfer value: 
	yearEpoch 			= The year of the epoch, so z. B. 1900 as integer 
	actualDay 			= Calculation day as integer 
	actualMonth 			= Calculation month as integer 
	actualYear 			= Calculation year as integer 
	
	Return value: 
	days 					= Number of days from the beginning of the epoch
	false						= On error
	******************************************************************************************* */	
function calcDaysSinceEpoch(yearEpoch, actualDay, actualMonth, actualYear){
	var YEAREPOCH = parseInt(yearEpoch);			// Das Jahr der Epoche ... z.B. 2010
																							// The year of the epoch ... e.g. 2010
	var ACTUALDAY = parseInt(actualDay);				// Aktueller Tag
																							// Actual day
	var ACTUALMONTH = parseInt(actualMonth);	// Aktueller Monat
																							// Actual month
	var ACTUALYEAR = parseInt(actualYear);			// Aktuelles Jahr
																							// Actual year
	var year = 0;																	// Hilfsvariablen fr Schleifen
	var years = 0;																// Auxiliary variables for loops
	var days = 0.0;																// Ergebnis
																							// Result
	// Jahr der Epoche und aktuelles Jahr sind gleich ...
	// The year of the epoch and the current year are the same. . .
	if (YEAREPOCH == ACTUALYEAR){
		days = convertDateToDaycount (ACTUALDAY, ACTUALMONTH, ACTUALYEAR);
	// Es gibt eine Differenz zwischen Epoche und aktuellem Jahr ...
	// ... und die Tage mssen aufaddiert werden.
	// There is a difference between epoch and current year . . . 
	// . . . and the days must be added up.
	} else if (YEAREPOCH < ACTUALYEAR){
		// Durchlaufe die Jahre von Beginn der Berechnungsgrudlage
		// der Epoche und addiere die Jahre auf.
		// Run through the years from the beginning of the calculation rudiment
		// of the epoch and add up the years.
		for(year = YEAREPOCH; year < ACTUALYEAR; year++){
			// Addiere die Tage des Jahres auf ...
			// Add the days of the year to . . .
			days += convertDateToDaycount (31, 12, year);
		}
		// ... und dann noch die Tage des aktuellen ...
		// . . . and then the days of the current ...
		days += convertDateToDaycount (ACTUALDAY, ACTUALMONTH, year);
	// ... das aktuelle Jahr liegt vor der Berechnungsgrundlage der Epoche.
	// . . . the current year lies before the calculation base of the epoch.
	} else if (YEAREPOCH > ACTUALYEAR) {				
		// Vertausche Epoche und das aktuelle Jahr in der Schleife
		// Durchlaufe die Jahre von Beginn der Berechnungsgrundlage
		// der Epoche und addiere die Jahre auf.
		// Swap epoch and the current year in the loop 
		// Run through the years from the beginning of the calculation basis 
		// of the epoch and add up the years.
		for(year = YEAREPOCH; year > ACTUALYEAR; year--){	
			// Addiere die Tage des Jahres auf ...
			// Add the days of the year to . . .
			days -= convertDateToDaycount (31, 12, year);				
		}
		// ... und dann noch die Tage des aktuellen ...
		// . . . and then the days of the current ...
		days += convertDateToDaycount (ACTUALDAY, ACTUALMONTH, year);			
	// Alle anderen Flle und Fehlercode -1.
	// All other cases and error code -1.
	} else {
		days = false;
	}
	// Tage zurck zum Aufrufer
	// Days bach to the caller
	return (days);
	
}// End of calcDaysSinceEpoch()


					

 /* *********************************************************************************************
	Die Funktion korregiert die Anzahl der Tage seit Epochenbeginn solange bis die
	Gradzahl im Bereich 0 - 360  liegt. Das ist die Hilfsgroesse N.
	Die bergebenen Tage werden dabei auf die Lnge des tropischen Jahres = 365,24219052
	korregiert.
	In eine Jahr bewegt sich die Erde ja um 360 um die Sonne. Somit sind die Anzahl
	der Tage seit Epochenbeginn = Ma fr die Gradzahl mit der sich die Erde um die
	Sonne weiter gedreht hat. Dies kann +/- xxxxx.yyyyy Grad sein. Um das Ergebnis auf die
	augenblickliche Position um die Sonne zu bekommen muss man das Ergebnis
	in den Bereich +/- 0 - 360 bewegen.
	
	bergabewert:
	d			= 	Die Anzahl der Tage seit Epochenbeginn als Integer
							
	Rckgabewert:
	N			= 	Hilfsgre N zur Berechnung der mittleren Anomalie M bei kreisrunder
					Umlaufbahn
	----------------------------------------------------------------------------------------------------------
	The function corrects the number of days since the beginning of the epoch until the
	Grade number is in the range 0-360. This is the auxiliary size N.
	The days passed are calculated on the length of the tropical year = 365,24 219 052
	corrected.
	In one year the earth moves 360 around the sun. So the number of days since 
	the beginning of the epoch = measure for the number of degrees with which the 
	earth has rotated around the sun. This can be +/- xxxxx. yyyyy degrees. To get the 
	result on the current position around the sun you have to move the result in the 
	range +/- 0 - 360.
	
	Transfer value:
	d 			= The number of days since the beginning of the epoch as an integer

	Return value:
	N 			= auxiliary size N for calculating the mean anomaly M in circular circles
				   Orbit
	******************************************************************************************* */	
function calculateN(d){
	var D = Number(d);	// Anzahl der Tage seit Epochenbeginn
											// Count days since epoch starts
	var N = 0.0;					// Ergebnis - Solution
	// Berechne die Hilfsgre N
	// Calculate the auxiliary variable N
	N = (360.0 / 365.242191) * D	;
	// Positiver Wert, dann 360 subtrahieren
	// Positive value, then substract 360
	// Negativer Wert, dann 360 addieren
	// Negative value, then add 360
	N = unwindDeg(N);
	// Korregierte Tage an Aufrufer zurck
	// Corregated Days to Callers Back
	return(N);
	
}// End of calculateN()
					

					
					
/*	*********************************************************************************************
	Die Funktion korregiert die mit der Funktion mediumSun() gefundene
	gesamt Gradzahl aller Umlufe um die Sonne und der mit calculateN()
	gefundene aktuelle Position, mit der mittleren ekliptischen Lnge epsilong
	und dem ekliptischen Lngengrad der Sonne omegag, und ermittelt daraus
	die mittlere Anomalie der Sonne.
	
	Quelle:
	https://de.wikipedia.org/wiki/Kepler-Gleichung#Mittlere_Anomalie
	
	bergabewert:
	N													= 	Aktuell berechnete Position in Grad 0 - 360
															als Number
	epsilong										=	Die mittlere eklipt. Lngengrad der Sonne
	omegag										=	Der eklipt. Lngengrad der Sonne im Perigum
	
	Rckgabewert:
	Mittlere Anomalie Sonne				= 	Als aktuelle Position in Grad bei kreisfrmigem Orbit
															0 - 360
	----------------------------------------------------------------------------------------------------------
	The function corrects the total number of degrees of all orbits around the sun found 
	with the function mediumSun() and the current position found with calculateN(), 
	with the mean ecliptic longitude epsilong and the ecliptic longitude of the sun omegag, 
	and calculates the mean anomaly of the sun. 
	
	Source: 
	https://de. wikipedia. org/wiki/Kepler-Gleichung#Mittlere_Anomalie 
	
	Transfer value: 
	N 								= Current calculated position in degrees 0 - 360 as Number 
	epsilong 					= The middle eklipt. Longitude of the sun 
	omegag 					= The ecliptic. Longitude of the Sun at perigee 
	
	Return value: 
	
	Mean anomaly Sun 	= As current position in degrees with circular orbit 0 - 360
	******************************************************************************************* */		
function mediumSun2(N, epsilong, omegag){
	var n = Number(N);								// Die in calculateN() gefundene Position 0 - 360
																	// The position found in calculateN () 0  360
	var m = 0.0;												// Mittlere Anomalie als Ergebnis
	var EPSILONG = Number(epsilong);		// Die mittlere eklipt. Lnge der Sonne
																	// The middle eclipt. Length of the Sun
	var OMEGAG = Number(omegag);		// Der eklipt. Lngengrad der Sonne im Perigum
																	// The eclipt. Longitude of the Sun in the Perigaeum
	// Das Ergebnis hier mit den beiden Parametern berechnen	
	// Calculate the result here with the two parameters	
	m = n + EPSILONG - OMEGAG;
	// Bei negativer Gradzahl 360 addieren		
	// Add 360 at negative degree
	m = unwindDeg(m);
	// ... und das Ergebnis der nittleren Anomalie an den Aufrufer zurck.
	// . . . and the result of the nitrous anomaly back to the caller.
	return (m);
	
}// End of mediumSun2()
					


					
// Berechnet eine nicht weiter benannte Hilfsgrsse
// Calculates a no further specified auxiliary size
function calculateEc(e, msonne){
	var E = Number(e);							// Die Exzentrititt der ellipt. Umlaufbahn
																// The eccentricity of the elliptical. Orbit
	var MSONNE = Number(msonne);	// Mittlere Anomalie der Sonne
																// Medium anomaly of the Sun
	var RAD = 180.0 / Math.PI;					// Umrechnung in Radiant
																// Conversion to radians
	// Ergebnis Berechnen und zurck an der Aufrufer.
	// Calculate result and return to the caller.
	return((360.0/Math.PI) * E * Math.sin(MSONNE / RAD));
	
}// End of calculateEc()

				
					
// Berechnet den ekliptischen Lngengrad (Lamda) der Sonne
/*	*********************************************************************************************
	Die Funktion berechnet den ekliptischen Lngengrad (Lamda)der Sonne. der
	ekliptische Breitengrad der Sonne ist ja 0.0, da sie in der Ekliptik liegt.
	
	Quelle:
	https://de.wikipedia.org/wiki/Astronomische_Koordinatensysteme
	
	bergabewert:
	N													= 	Aktuell berechnete Position in Grad 0 - 360
															als Number
	ec													= 	Korrekturwert
	epsilong										=	Die mittlere eklipt. Lngengrad der Sonne

	
	Rckgabewert:
	Ekliptischer Lngengrad Sonne		= 	Als aktuelle Position in Grad bei kreisfrmigem Orbit
															0 - 360
	----------------------------------------------------------------------------------------------------------
	The function calculates the ecliptic longitude (Lamda) of the sun. the ecliptic 
	latitude of the sun is 0. 0, since it lies in the ecliptic. 
	
	Source: https://de. wikipedia. org/wiki/Astronomische_Koordinatensysteme 
	
	Transfer value: 
	N 													= Current calculated position in degrees 0 - 360 as Number 
	ec 												= Correction value 
	epsilong 										= The mean eklipt. Longitude of the sun 
	
	Return value: 
	Ecliptic longitude of the sun 			= As current position in degrees for circular orbit 0 - 360
	******************************************************************************************* */
function lambdaSun(n, ec, epsilong){
	var N = Number(n);							// Unkorregierte mittlere Anomalie der Sonne in Grad
	var EC = Number(ec);						// Korrekturwert
	var EPSILONG = Number(epsilong);	// Die mittlere eklipt. Lnge der Sonne
	var ldsun = 0.0;									// Ekliptischer Lngengrad der Sonne als Ergebnis
	// bergabewerte aufaddieren ...
	ldsun = N + EC + EPSILONG;
	// Das Ergebnis in den Bereich 0 - 360 bringen ...
	ldsun = unwindDeg(ldsun );
	// ... und zurck an den Caller.
	return(ldsun);
	
}// End of lambdaSun()

					
/* ******************************************************
	Die Funktion Konvertiert Winkelsekunden in
	dezimale Grad als Bruchteil einer Winkelstunde.
						
	bergabewert:
	arcsec						= 	Winkelsekunde als Ganzzahl, auch mit
											Nachkommaanteil, wie 35.67 " 
							
	Rckgabewert:
	Winkelsekunde			= 	Die Winkelsekunde als Dezimalwert
											und Bruchteil einer Winkelstunde
	--------------------------------------------------------------
	The function Converts angular seconds to
	decimal degrees as a fraction of an angular hour.
	
	Transfer value:
	arcsec 						= 	Angular second as an integer, also with
										Decimal percentage, as 35. 67 "

	Return value:
	Angular second 		= 	The angular second as a decimal value
										and a fraction of an angular hour
	*************************************************** */						
function convertAngelsecondsToDegrees(arcsec){
	var ARCSEC = Number(arcsec);
	
	return(ARCSEC / (3600.0));
	
}// End of convertAngelsecondsToDegrees()
					
		
		
/* ******************************************************
	Die Funktion Konvertiert dezimale Grad zu 
	einem String als Grad  Minute ' Sekunde ".
	Z.B. 134 34' 56.7"
						
	bergabewert:
	decdegrees			= 	Grad in dezimalem 
										Format z.B. 134.238157 als Number
							
	Rckgabewert:
	strDegrees				= 	Der Ergebnisstring im
										oben beschriebenen Format
	--------------------------------------------------------------
	The function Converts decimal degrees to
	a string as a degree  minute' second. 
	E. g. 134 3456. 7.

	Transfer value:
	decdegrees 			= 	Grad in decimal
										Format e. g. B. 134. 238 157 as number

	Return value:
	strDegrees 			= The result string in
										above described format
	
	*************************************************** */					
function convertDecdegreesToString(decdegrees){
	var DECDEGREES = Number(decdegrees);	// bergabewert zu Number
																			// Tranfer value to Number
	var rest = 0.0;													// Variable des Dezimalanteils
																			// Variable of the decimal part
	var strDegrees = "";											// Ergebnisstring Initialisieren
																			// Initialize result string
	// Bei negativen Graden muss ceil()verwendet werden, das das
	// Ergebnis ansonsten um 1 kleiner ist ...
	// In case of negative grades, ceil () must be used, which
	// Otherwise the result is 1 smaller. . .
	if (DECDEGREES  <  0.0){
		// ... und holen den ganzen Anteil des Grades ...
		// . . . and get the whole part of the degree . . .
		strDegrees = strDegrees + Math.ceil(DECDEGREES) + " ";
		// ... und den dezimalen Anteil in rest speichern ...		
		//. . . and save the decimal share in rest. . .
		rest = (DECDEGREES - Math.ceil(DECDEGREES)) * -1;
	// Bei positiven Graden benutze floor(), ansonsten ist das Ergebnis
	// um 1 grer ...
	// For positive grades use floor (), otherwise the result is
	// by 1 larger . . .
	}else{
		// ... und holen den ganzen Anteil des Grades ...
		// . . . and get the whole part of the degree . . .
		strDegrees = strDegrees + Math.floor(DECDEGREES) + " ";
		// ... und den dezimalen Anteil in rest speichern ...	
		//. . . and save the decimal share in rest. . .
		rest = DECDEGREES - Math.floor(DECDEGREES);
	}
	// ... die ganzen Minuten aus dem Rest der Grade berechnen 
	// und im Ergebnisstring speichern ...
	//. . . calculate the whole minutes from the rest of the grades
	// and save in the result string. . .
	strDegrees = strDegrees + Math.floor(rest * 60.0) + "' ";
	// ... dann Nachkommaanteil der Minuten ermitteln ...
	//. . . then determine decimal part of the minutes. . .
	rest = (rest * 60.0)- Math.floor(rest * 60.0);
	// ... und die Sekunden aus dem Rest der Minuten ermitteln ...
	// . . . and determine the seconds from the rest of the minutes . . .
	strDegrees = strDegrees + Math.floor(rest * 60.0 ) + "\"";
	// ...	und zurck an den Caller.		
	//. . . and back to the caller.
	return(strDegrees);
	
}// End of convertDecdegreesToString()




/* ******************************************************
	Die Funktion berechnet den Wert epsilon fr die 
	ekliptik Schiefe.	Das ist der Winkel zwischen 
	der Ekliptik im Sonnensystem und dem quator
	der Erde.
	
	Quelle:
	https://de.wikipedia.org/wiki/Ekliptik#Die_Schiefe_der_Ekliptik
						
	bergabewert:
	jd						= 	Julianisches Datum des Berechnungsdatums als Number
							
	Rckgabewert:
	epsilon				= 	Ekliptische Schiefe in Grad und als Number
	--------------------------------------------------------------
	The function calculates the value epsilon for the ecliptic 
	skewness. 
	This is the angle between the ecliptic in the solar system 
	and the equator of the earth. 
	
	Source: https://de. wikipedia. org/wiki/Ekliptik#Die_Schiefe_der_Ekliptik 
	
	Transfer value: 
	jd 						= Julian date of the calculation date as Number 
	
	Return value: 
	epsilon 				= Ecliptic skew in degrees and as Number
	*************************************************** */		
function calcObliquityEcliptic(jd){
	var JD = Number(jd);		// Julian date of calculation
												// Julianisches Berechnungsdatum
	var T = 0.0;							// Teil des Jahrhunderts der Epoche
												// Part of the century of the epoch
	var epsilon = 0.0;					// Ekliptische Schiefe
												// Ecliptic obliquity
	var DE = 0.0;						// Hilfsvariable
												// Auxilary variable
	var CENTURY = [];				// Der vergangene Teil des Jahrhunderts als
												// Array in CENTURY[1]
												// Part of the century in an array CENTURY[1]
	// Jahrhundert seit 1. Januar 2000 12:00 Uhr
	CENTURY = partOfTheCentury(2000, JD);
	T = CENTURY[1];
	// Hilfsgre in Winkelsekunden ...
	DE = 46.815 * T + 0.0006 * Math.pow(T, 2) + 0.00181 * Math.pow(T, 3);
	// ... und nach Grad konvertieren ...
	DE /= 3600.0;
	// ... epsilon in Winkelsekunden berechnen ...
	epsilon = 84381.45 - 46.815 * T - 0.0006 * Math.pow(T, 2) + 0.00181 * Math.pow(T, 3);
	// DE noch von epsilon subtrahieren.
	epsilon -= DE;
	// ... Winkelsekunden in dezimale Grad umrechnen ...
	epsilon = convertAngelsecondsToDegrees(epsilon);
	// ... und zurck an den Caller.
	return(epsilon);
						
}// End of calcObliquityEcliptic()
					
					
					
					
/* *************************************************************************
 * Konvertiert eine dezimale Gradangabe in einen Stundenwinkel als
 *  ' ", also z. B. 144.0424999999999 -> 144 02' 33''
 *	
 * bergabewert:
 * decDegrees = Die dezimale Gradangabe als Number
 *	
 * Returnwert:
 * Die Stundenangabe in x y' z" als String
 * 	-----------------------------------------------------------------------------------
 * Converts a decimal degree to an hourly angle as
 *  also e. g. 144. 0 424 999 999 999 -> 144 02' 33''.
 *
 * Transfer value:
 * decDegrees = The decimal degree as number
 *
 * Return value:
 * The hours given in x y' z'' as a string.
	********************************************************************* */	
function decDegrees2AngleOfHours(decDegrees){
	var DECDEGREES = Number(decDegrees);	// Die dezimale Gradangabe als Number
																			// The decimal degree as number
	var ANGLEOFHOURS = "";								// Ergebnis als String
																			// Solution as string
	var rest = 0.0;													// Hilfsvariable
																			// Auxilary variable
	// Der ganzzalige Anteil der Dezimalzahl sind die  bzw. die
	// anteiligen Stunden, denn 360 sind 24 Stunden eines Tages und dies dem
	// String zuordnen ...
	// The total proportion of the decimal number is the  or the
	// proportional hours, because 360 is 24 hours a day and this is the
	// Assign string. . .
	ANGLEOFHOURS = ANGLEOFHOURS + Math.floor(DECDEGREES) + " ";
	// ... jetzt den Nachlommaanteil aus der Dezimalzahl und dem
	// ganzzahligen Anteil ermitteln ...
	// . . . now calculate the fractional part from the decimal number and the 
	// integer part . . .
	rest = DECDEGREES - Math.floor(DECDEGREES);
	// ... der Rest x 60 sind die Minuten, denn 1 Stunde hat 60 Minuten
	// und daraus den ganzzahligen Anteil ermitteln ...
	// . . . the rest x 60 are the minutes, because 1 hour has 60 minutes
	// and determine from it the integer part . . .
	ANGLEOFHOURS = (ANGLEOFHOURS + LPad(Math.floor(rest * 60.0), 2, "0")) + "' ";
	// Der gerundete Wert des ganzzahlige Anteil des rest * 60.0 sind 
	// dann die '' Minuten ...
	// The rounded value of the integer part of the rest * 60. 0 are 
	// then the '' minutes . . .
	ANGLEOFHOURS = ANGLEOFHOURS + (((rest * 60.0) - Math.floor(rest * 60.0)) * 60.0).toFixed(2) + "''";
	// ... und zurck zum Aufrufer.
	//	... and back to the caller.
	return(ANGLEOFHOURS);
	
} // End of decDegrees2AngleOfHours()


					
					
/* *****************************************************************************
 * Konvertiert eine dezimale Gradangabe der RA in einen Winkelzeiten als
 * hh mm ss.ss, also z. B. 144.0424999999999 -> 10h 02m 33.35s
 * Macht die umgekehrte Rechnung wie angleOfTime2DecDegrees(
 * angleoftimearray).
 *	
 * bergabewert:
 * Die RA in dezimalen Grad als Number
 *	
 * Returnwert:
 * Die Winkelzeiten in hh mm ss.ss als Array mit dem Index 0 - 2.
 * 0 = Stunden
 * 1 = Minuten
 * 2 = Sekunden
 *  --------------------------------------------------------------------------------------
 * Converts a decimal degree of the RA to an angle time as
 * hh mm ss. ss, e. g. 144. 04 249 999 999 -> 10h 02m 33. 35s
 * Makes the reverse calculation like angleOfTime2DecDegrees (
 * angleoftimearray). *
 * Transfer value:
 * The RA in decimal degrees as number
 *
 * Return value:
 * The angular times in hh mm ss. ss as an array with the index 0-2. *0 = hours
 *1 = minutes
 *2 = seconds
	************************************************************************* */	
function decDegrees2AngleOfTime(decDegrees){
	var DECDEGREES = Number(decDegrees);	// bergabewert Gradwert nach Number
																			// Transfer value Degree value according to number
	var ANGLEOFTIME = [];									// Leeres Returnarray
																			// Empty Returnarray
	var rest = 0.0;													// Fr Zwischenrechnungen
																			// For interim invoices
	var HOURS = 0.0;											// Winkelstunden
																			// Angle hours
	var MINUTES = 0.0;											// Winkelminuten
																			// Angle minutes
	var SECONDS = 0.0;										// Winkelsekunden
																			// Angle seconds
	// Zuerst die Stunden aus den Grad / 15 berechnen, 360 = 24h 
	// First calculate the hours from the degrees / 15, 360 = 24h
	HOURS = LPad(Math.floor(DECDEGREES/15.0), 2, "0");
	ANGLEOFTIME.push(HOURS);
	// ... jetzt den Nachlommaanteil aus der Dezimalzahl und dem
	// ganzzahligen Anteil ermitteln und diesen mit 15.0 Multiplizieren,
	// dann bekommt man den Rest in Grad ...
	//. . . now the afterlomma part of the decimal number and the
	// determine the integer part and multiply it by 15. 0,
	// then you get the rest in degrees . . .
	rest = ((DECDEGREES/15.0) - Math.floor(DECDEGREES/15.0)) * 15.0;	
	// ... nun diese restlichen Grad durch 0.25 Grad/Minute dividieren
	// und man bekommt mit dem ganzzahligen Anteil die Minuten ...
	//. . . now divide these remaining degrees by 0. 25 degrees/minute
	// and you get with the integer part the minutes . .
	MINUTES = LPad(Math.floor(rest/0.25), 2, "0");
	ANGLEOFTIME.push(MINUTES);	
	// ... jetzt aus den Nachkomma Minuten wieder die restlichen Grad
	// ermitteln, indem man mit 0.25 Grad/Minute multipliziert ...
	// . . . now from the decimal point minutes again the remaining degrees
	// Determine by multiplying by 0. 25 degrees/minute. . .
	rest = ((rest/0.25) - MINUTES) * 0.25;			
	// ... die restlichen Grad nun durch 0.00416666 Grad/Sekunde dividieren
	// und man erhlt die verbliebenen Sekunden.
	// . . . now divide the remaining degrees by 0. 00 416 666 degrees/second
	// and you get the remaining seconds.
	SECONDS = LPad( (rest/0.004166666666667).toFixed(2), 2, "0");
	ANGLEOFTIME.push(SECONDS);
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(ANGLEOFTIME);
	
} // End of decDegrees2AngleOfTime()


			
					
/* *************************************************************************
 * Konvertiert einen Stundenwinkel im Format hh mm ss.ss in eine dezimale
 * Winkelangeb, wie 144.234567 Grad.
 * Macht die umgekehrte Rechnung wie decDegrees2AngleOfTime(decDegrees).
 *
 * bergabewert:
 * Ein Array mit den idx 0 - 2 und den Werten des Stundenwinkels
 * 0 = Stunden
 * 1 = Minuten
 * 2 = Sekunden
 *	
 * Returnwert:
 * Der Winkel in Grad als Number.
 *  --------------------------------------------------------------------------------------
 * Converts an hour angle in the format hh mm ss. ss to a decimal 
 * angle, such as 144. 234567 degrees. 
 * Does the reverse calculation like decDegrees2AngleOfTime(decDegrees). 
 * 
 * Transfer value: 
 * An array with the idx 0 - 2 and the values of the hour angle 
 * 0 = hours 
 * 1 = minutes 
 * 2 = seconds 
 * 
 * Return value: 
 * The angle in degrees as number.
	************************************************************************* */	
function angleOfTime2DecDegrees(angleoftimearray){
	var DECDEGREES = 0.0;									// Returnwert als Number und dezimale Grad
																			// Return value as number and decimal degrees
	var DEGREESPERSECOND = 15.0 / 3600.0;	// Die Grad pro Sekunde bei 15 = 1h = 3600s
																			// The degrees per second at 15 = 1h = 3600s
	// ... Zuerst die vollen Grad aus den h im Array bei 0 berechnen 
	// in dem man die h mit 15/h multipliziert ...
	//. . . First calculate the full degrees from the h in the array at 0
	// where you multiply the h by 15/h. . .
	DECDEGREES += parseInt(angleoftimearray[0]) * 15.0;
	// dazu die Grad aus den Minuten multipliziert mit 0.25 Grad/Minute addieren ...
	// add the degrees from the minutes multiplied by 0. 25 degrees/minute. . .
	DECDEGREES += parseInt(angleoftimearray[1]) * 0.25;			
	// ... dazu die Grad aus den Sekunfen multipliziert mit 0.0041666.. Grad/Sekunde
	// addieren.
	// . . . add the degrees from the seconds multiplied by 0. 0041666. . add degrees/second.
	DECDEGREES += Number(angleoftimearray[2]) * DEGREESPERSECOND;
	// ... und zurck zum Aufrufer.
	// ... and back to the caller.
	return(DECDEGREES);
	
}// End of angleOfTime2DecDegrees()


					

/* *************************************************************************
 * Berechnet die Nutation fr die geografische Lnge delta Psi der 
 * Sonne und der Ekliptik Schiefe delta Epsilon.
 *
 * bergabewert:
 * jd					= Julianisches Datum der Berechnung
 * epoch			= Die Epoche, z.B. 1900, 2000, 2010 .....
 *	
 * Returnwert:
 * Array
 * NUTATION[0] = Nutation fr ekliptische Lnge der Sonne in ''
 * NUTATION[1] = Nutation fr ekliptische Schiefe in ''
 *  -------------------------------------------------------------------------------------
 * Calculates the nutrition for the geographical length delta Psi of the
 * Sun and the Ecliptic Slope delta Epsilon. *
 * Transfer value:
 * jd
 * epoch = The Epoch, e. g. B. 1900, 2000, 2010 . . . . .
 *
 * Return value:
 * Array
 *NUTATION[0] = Nutation for ecliptic length of the Sun in "
 *NUTATION[1] = Nutation for ecliptic slopes in "
	************************************************************************* */	
function calcNutation(jd, epoch){
	var EPOCH = Number(epoch); // Die verwendete Epoche
														// Used epoch
	var NUTATION = [];						// Array fr die beiden Werte der Nutation
														// Array for the two values of the nutation
	var T = 0.0;									// Der Teil des vergangenen Jahrhunderts seit der Epoche
														// The part of the last century since the epoch
	var JD = Number(jd);				// Das julian. Datum des Berechnungsdatums
														// The julian. Date of calculation
	var L = 0.0;									// Die Mittlere Lnge der Sonne
														// The Medium Length of the Sun
	var A = 0.0;									// Hilfsvariable fr L
														// Auxilary variable for L
	var B = 0.0;									// Hilfsvariable fr Omega, dem ansteigenden Knoten
														// des Mondes
														// Auxiliary variable for Omega, the rising node
														// of the moon
	var OMEGA = 0.0;						// Ansteigender Knoten des Mondes in Grad
														// Rising Node of the Moon in degrees
	var RAD = 180.0 / Math.PI;			// Umrechnung in Radiant
														// Conversion to radian
	var DELTAPSI = 0.0;					// Nutation in der geografischen Lnge
														// Nutation in the geographical longitude
	var DELTAEPSILON = 0.0;			// Nutation der ekliptischen Schiefe
														// Nutation of the ecliptic obliquity
	// Das vergangene Jahrhundert seit Epoche 2010 als Array ...
	// in [0] steht das verwendete Jahrhundert, in [1] der Teil des
	// vergangenen Jahrhunderts der Epoche ...
	// The past century since the epoch of 2010 as an array . . .
	// in [0] is the century used, in [1] the part of the
	// last century of the epoch . . .
	CENTURY = partOfTheCentury(EPOCH, JD);
	T = CENTURY[1];
	// ... Hilfswert ...
	// ... Auxilary value ...
	A = 100.002136 * T;
	// ... dann die mittlere Lnge der Sonne L berechnen ...
	//. . . then calculate the mean length of the Sun L. . .
	L = 279.6966778 + (360.0 * (A - Math.floor(A)));
	// ... L in den Bereich 0 - 360 bringen ...
	//. . . Bring L to the range 0  360. . .
	L = unwindDeg(L);
	// ... nun Omega als ansteigenden Knoten des Mondes in Grad berechnen ...
	//. . . now calculate omega as the rising node of the moon in degrees. . .
	B = 5.372617 * T;
	OMEGA = 259.1833 - 360.0 * (B - Math.floor(B));
	// ... OMEGA in den Bereich 0 - 360 bringen ...
	//. . . Bring OMEGA to the range 0  360. . .
	OMEGA = unwindDeg(OMEGA);				
	// ... dann Delta Psi in Bogensekunden als geografische Lnge der Sonne berechnen ...
	//. . . then calculate Delta Psi in arc seconds as the geographical length of the sun. . .
	DELTAPSI = (-17.2 * Math.sin(OMEGA/RAD)) - (1.3 * Math.sin((2*L)/RAD));
	NUTATION.push(DELTAPSI);		
	// ... und die Nutation in Bogensekunden der ekliptischen Schiefe berechnen ...
	//. . . and calculate the nutation in arc seconds of the ecliptic slope. . .
	DELTAEPSILON = (9.2 * Math.cos(OMEGA/RAD)) + (0.5 * Math.cos((2*L)/RAD));
	NUTATION.push(DELTAEPSILON);
	// ... und Array an den Caller zurck.
	// NUTATION[0] = Nutation fr ekliptische Lnge der Sonne in ''
	// NUTATION[1] = Nutation fr ekliptische Schiefe in ''
	//. . . and Array back to the caller. // NUTATION[0] = Nutation for ecliptic length of the Sun in "
	// NUTATION[1] = Nutation for ecliptic slopes in "
	return(NUTATION);
	
}// End of calcNutation()